from datetime import datetime, time, timedelta
from timetable import (load_timetable, cut_timetable,
        merge_intersections, annotate_timetable, collect_keys)
# >>>
start, end = datetime(2015, 1, 1), datetime(2015, 2, 1)
calendar_files = {'Work': open('data/work.ics', 'rb')}
timetable = load_timetable(calendar_files, clip_start=start, clip_end=end)

# The :func:`~timetable.timetable.load_timetable` function supports loading
# and merging multiple calendars at once, which is why a dictionary must be
# passed in containing the calendar files. Furthermore, the function also
# annotates entries with tags. If a calendar entry does not contain a tag,
# the key of the dictionary is used by default.

# .. note::

# It is a good idea to supply at least a *clip_end* date. Otherwise, an
# infinite amount of entries might get generated, if there are unlimited
# recurring events.

days = []
day = start
while day < end:
    days.append(datetime.combine(day, time()))
    day += timedelta(days=1)

# The next step is to compute a list of :class:`~datetime.datetime` objects
# at which we want to cut the timetable entries.

# Cut timetable entries at day boundaries.
work = {}
for idx, day_timetable in enumerate(cut_timetable(timetable, days)):
    # Merge all intersecting entries.
    day_timetable = merge_intersections(day_timetable)
    # Collect tags from merged entries.
    day_timetable = annotate_timetable(day_timetable, collect_keys('tags'))
    for start, end, entry in day_timetable:
        # Compute duration and apply to timeseries.
        duration = (end - start).total_seconds() / len(entry['tags'])
        for tag in entry['tags']:
            if not tag in work:
                work[tag] = [0 for day in days]
            work[tag][idx] += duration

# The separation of a timetable into daily timetables is done by the function
# :func:`~timetable.timetable.cut_timetable`.

# Entries of a daily timetable might overlap. The
# :func:`~timetable.timetable.merge_intersections` function merges entries
# from a timetable based on a key and generates non-intersecting entries.

# The duration of an entry may now be calculated by simply subtracting the
# start datetime from the end datetime. The duration is accounted equally for
# each tag.

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import Patch
from matplotlib.cm import ScalarMappable
# >>>
fig = plt.figure()
ax = fig.add_subplot(1, 1, 1)
# >>>
# Draw a stackplot with custom colors and legend handles.
colormap = ScalarMappable(cmap='Paired')
colormap.set_clim(0, len(work))
data, colors, handles, labels = [], [], [], []
for idx, name in enumerate(sorted(work)):
    color = colormap.to_rgba(idx)
    data.append(np.cumsum(work[name]) / 3600)
    colors.append(color)
    handles.append(Patch(color=color))
    labels.append(name)
ax.stackplot(days, data, colors=colors)
# [...]
# >>>
fig.autofmt_xdate()
ax.set_ylabel('Hours')
# <...>
ax.legend(handles, labels, loc='upper left')
# <...>
# >>>
plt.show() # doctest: +SKIP

# The last code fragment builds a stacked cumulative plot for each of the
# collected time series.
